==> picolibc-1.8.6.tar.xz: picolibc-1.8.6/newlib/libc/tinystdio/fdopen.c <==
/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2019 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdio_private.h"

FILE *
fdopen(int fd, const char *mode)
{
	int stdio_flags;
	int open_flags;
	struct __file_bufio *bf;
        char *buf;

	stdio_flags = __posix_sflags(mode, &open_flags);
	if (stdio_flags == 0)
		return NULL;

	/* Allocate file structure and necessary buffers */
	bf = calloc(1, sizeof(struct __file_bufio) + BUFSIZ);

	if (bf == NULL) {
		close(fd);
		return NULL;
	}
        buf = (char *) (bf + 1);

        *bf = (struct __file_bufio)
                FDEV_SETUP_POSIX(fd, buf, BUFSIZ, stdio_flags, 0);

        __bufio_lock_init(&(bf->xfile.cfile.file));

	if (open_flags & O_APPEND)
                (void) fseeko(&(bf->xfile.cfile.file), 0, SEEK_END);

	return &(bf->xfile.cfile.file);
}

==> picolibc-1.8.10.tar.xz: picolibc-1.8.10/newlib/libc/tinystdio/fdopen.c <==
/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2019 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdio_private.h"

FILE *
fdopen(int fd, const char *mode)
{
	int stdio_flags;
	int open_flags;
	struct __file_bufio *bf;
        char *buf;

	stdio_flags = __stdio_flags(mode, &open_flags);
	if (stdio_flags == 0)
		return NULL;

	/* Allocate file structure and necessary buffers */
	bf = calloc(1, sizeof(struct __file_bufio) + BUFSIZ);

	if (bf == NULL) {
		close(fd);
		return NULL;
	}
        buf = (char *) (bf + 1);

        *bf = (struct __file_bufio)
                FDEV_SETUP_POSIX(fd, buf, BUFSIZ, stdio_flags, __BFALL);

	if (open_flags & O_APPEND)
                (void) fseeko(&(bf->xfile.cfile.file), 0, SEEK_END);

	return &(bf->xfile.cfile.file);
}
